CONNECTIONS_ERRORS = [
    "failed",
    "denied",
    "error",
    "closed",
    "invalid",
    "disabled",
    "Bad",
    "Bad configuration",
    "refused",
    "failure",
    "Could not",
    "Connection reset by peer",
    "Received disconnect",
    "No supported authentication methods available",
    "Unable to negotiate with",
    "Connection timed out",
    "Broken pipe",
    "Network is unreachable",
    "Host is down",
    "Connection reset by [host]",
    "Resource temporarily unavailable",
    "Protocol major versions differ",
    "Protocol mismatch",
    "Connection aborted",
    "SSH protocol version mismatch",
    "Host identification has changed",
    "Operation timed out",
    "Host is unreachable",
    "No route to host",
    "Connection aborted by peer",
    "SSH packet corrupt",
    "Connection lost",
    "Broken authentication sequence",
    "No space left on device",
    "Host not found",
    "Address already in use",
    "Connection timed out waiting for session",
    "Host public key mismatch",
    "Cannot bind to port",
    "Unsupported protocol version",
    "Unexpected EOF in input",
    "SSH connection dropped",
    "Server host key mismatch",
    "Authentication rejected by peer",
    "Host key does not match",
    "Max sessions reached",
    "Too many concurrent sessions",
    "Unexpected disconnection",
    "Host is blacklisted",
    "User not allowed by policy",
    "Received SSH_MSG_DISCONNECT",
    "SSH key expired",
    "Connection attempt rejected",
    "SSH agent not available",
    "RSA host key for has changed",
    "Fingerprint mismatch",
    "Unable to connect",
    "Host temporarily unavailable",
    "No response from host",
    "Read from socket failed: Connection timed out",
    "Timeout waiting for input",
    "Timeout before authentication",
    "Session timeout",
    "Client timeout",
    "Timeout while connecting",
    "Timeout expired",
    "Handshake timeout",
    "Operation timeout",
    "Login timeout",
    "Network timeout",
    "SSH connection timeout",
    "Request timeout",
    "Timed out waiting for response",
    "Connection timeout during handshake",
    "Timeout error during key exchange",
    "Socket timeout",
]
